@@ -139,7 +139,7 @@ def plot_as_array(
     #  call appropriate routine
 
     if ndim == 1:
-        return plot_as_array_1d(
+        coll, dax, dgroup = plot_as_array_1d(
             # parameters
             coll=coll2,
             key=key,
@@ -154,7 +154,6 @@ def plot_as_array(
             aspect=aspect,
             nmax=nmax,
             color_dict=color_dict,
-            dinc=dinc,
             lkeys=lkeys,
             bstr_dict=bstr_dict,
             rotation=rotation,
@@ -164,11 +163,10 @@ def plot_as_array(
             fs=fs,
             dcolorbar=dcolorbar,
             dleg=dleg,
-            connect=connect,
         )
 
     elif ndim == 2:
-        return plot_as_array_2d(
+        coll, dax, dgroup = plot_as_array_2d(
             # parameters
             coll=coll2,
             key=key,
@@ -185,7 +183,6 @@ def plot_as_array(
             aspect=aspect,
             nmax=nmax,
             color_dict=color_dict,
-            dinc=dinc,
             lkeys=lkeys,
             bstr_dict=bstr_dict,
             rotation=rotation,
@@ -196,11 +193,10 @@ def plot_as_array(
             fs=fs,
             dcolorbar=dcolorbar,
             dleg=dleg,
-            connect=connect,
         )
 
     elif ndim == 3:
-        return plot_as_array_3d(
+        coll, dax, dgroup = plot_as_array_3d(
             # parameters
             coll=coll2,
             key=key,
@@ -219,7 +215,6 @@ def plot_as_array(
             aspect=aspect,
             nmax=nmax,
             color_dict=color_dict,
-            dinc=dinc,
             lkeys=lkeys,
             bstr_dict=bstr_dict,
             rotation=rotation,
@@ -231,9 +226,23 @@ def plot_as_array(
             fs=fs,
             dcolorbar=dcolorbar,
             dleg=dleg,
-            connect=connect,
         )
 
+    # --------------------------
+    # add axes and interactivity
+
+    # add axes
+    for ii, kax in enumerate(dax.keys()):
+        harmonize = ii == len(dax.keys()) - 1
+        coll.add_axes(key=kax, harmonize=harmonize, **dax[kax])
+
+    # connect
+    if connect is True:
+        coll.setup_interactivity(kinter='inter0', dgroup=dgroup, dinc=dinc)
+        coll.disconnect_old()
+        coll.connect()
+
+    return coll
 
 # #############################################################################
 # #############################################################################
@@ -387,10 +396,16 @@ def _plot_as_array_check(
             cmap = 'viridis'
 
     # vmin, vmax
-    if vmin is None and diverging:
-        vmin = -max(abs(nanmin), nanmax)
-    if vmax is None and diverging:
-        vmax = max(abs(nanmin), nanmax)
+    if vmin is None:
+        if diverging:
+            vmin = -max(abs(nanmin), nanmax)
+        else:
+            vmin = nanmin
+        if vmax is None:
+            if diverging:
+                vmax = max(abs(nanmin), nanmax)
+            else:
+                vmax = nanmax
 
     # vmin, vmax
     if ymin is None:
@@ -551,7 +566,6 @@ def plot_as_array_1d(
     aspect=None,
     nmax=None,
     color_dict=None,
-    dinc=None,
     lkeys=None,
     bstr_dict=None,
     rotation=None,
@@ -561,7 +575,6 @@ def plot_as_array_1d(
     fs=None,
     dcolorbar=None,
     dleg=None,
-    connect=None,
 ):
 
     # --------------
@@ -671,7 +684,7 @@ def plot_as_array_1d(
                 ref=ref,
                 data='index',
                 dtype='xdata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -697,19 +710,7 @@ def plot_as_array_1d(
             bstr_dict=bstr_dict,
         )
 
-    # add axes
-    for kax in dax.keys():
-        coll.add_axes(key=kax, **dax[kax])
-
-    # increment dict
-
-    coll.setup_interactivity(kinter='inter0', dgroup=dgroup, dinc=dinc)
-
-    # connect
-    if connect is True:
-        coll.connect()
-
-    return coll
+    return coll, dax, dgroup
 
 
 # #############################################################################
@@ -737,7 +738,6 @@ def plot_as_array_2d(
     aspect=None,
     nmax=None,
     color_dict=None,
-    dinc=None,
     lkeys=None,
     bstr_dict=None,
     rotation=None,
@@ -748,7 +748,7 @@ def plot_as_array_2d(
     fs=None,
     dcolorbar=None,
     dleg=None,
-    connect=None,
+    interactive=None,
 ):
 
     # --------------
@@ -941,41 +941,67 @@ def plot_as_array_2d(
     # plot mobile part
 
     kax = 'matrix'
-    for kax in lkax:
+    if dax.get(kax) is not None:
         ax = dax[kax]['handle']
 
         # ind0, ind1
-        for ii in range(nmax):
-            lh = ax.axhline(
-                dataY[ind[1]], c=color_dict['X'][ii], lw=1., ls='-',
-            )
-            lv = ax.axvline(
-                dataX[ind[0]], c=color_dict['Y'][ii], lw=1., ls='-',
-            )
+        if dax.get('vertical') is not None:
+            for ii in range(nmax):
+                lh = ax.axhline(
+                    dataY[ind[1]], c=color_dict['X'][ii], lw=1., ls='-',
+                )
 
-            # update coll
-            kh = f'h{ii:02.0f}'
-            kv = f'v{ii:02.0f}'
-            coll.add_mobile(
-                key=kh,
-                handle=lh,
-                ref=refY,
-                data=keyY,
-                dtype='ydata',
-                ax=kax,
-                ind=ii,
-            )
-            coll.add_mobile(
-                key=kv,
-                handle=lv,
-                ref=refX,
-                data=keyX,
-                dtype='xdata',
-                ax=kax,
-                ind=ii,
-            )
+                # update coll
+                kh = f'h{ii:02.0f}'
+                coll.add_mobile(
+                    key=kh,
+                    handle=lh,
+                    ref=refY,
+                    data=keyY,
+                    dtype='ydata',
+                    axes=kax,
+                    ind=ii,
+                )
 
-        dax[kax].update(refx=[refX], datax=keyX, refy=[refY], datay=keyY)
+            # for ax clic
+            ax_refx = [refX]
+            ax_datax = [keyX]
+        else:
+            # for ax clic
+            ax_refx = None
+            ax_datax = None
+
+        # ind0
+        if dax.get('horizontal') is not None:
+            for ii in range(nmax):
+                lv = ax.axvline(
+                    dataX[ind[0]], c=color_dict['Y'][ii], lw=1., ls='-',
+                )
+
+                # update coll
+                kv = f'v{ii:02.0f}'
+                coll.add_mobile(
+                    key=kv,
+                    handle=lv,
+                    ref=refX,
+                    data=keyX,
+                    dtype='xdata',
+                    axes=kax,
+                    ind=ii,
+                )
+
+            # for ax clic
+            ax_refy = [refY]
+            ax_datay = [keyY]
+        else:
+            # for ax clic
+            ax_refy = None
+            ax_datay = None
+
+
+        dax[kax].update(
+            refx=ax_refx, datax=ax_datax, refy=ax_refy, datay=ax_datay,
+        )
 
     kax = 'vertical'
     if dax.get(kax) is not None:
@@ -999,7 +1025,7 @@ def plot_as_array_2d(
                 ref=(refX,),
                 data=key,
                 dtype='xdata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1014,7 +1040,7 @@ def plot_as_array_2d(
                 ref=(refY,),
                 data=keyY,
                 dtype='ydata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1042,7 +1068,7 @@ def plot_as_array_2d(
                 ref=(refY,),
                 data=[key],
                 dtype='ydata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1057,7 +1083,7 @@ def plot_as_array_2d(
                 ref=(refX,),
                 data=keyX,
                 dtype='xdata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1100,22 +1126,7 @@ def plot_as_array_2d(
             bstr_dict=bstr_dict,
         )
 
-    # --------------------------
-    # add axes and interactivity
-
-    # add axes
-    for kax in dax.keys():
-        coll.add_axes(key=kax, **dax[kax])
-
-    # increment dict
-
-    coll.setup_interactivity(kinter='inter0', dgroup=dgroup, dinc=dinc)
-
-    # connect
-    if connect is True:
-        coll.connect()
-
-    return coll
+    return coll, dax, dgroup
 
 
 # #############################################################################
@@ -1143,7 +1154,6 @@ def plot_as_array_3d(
     aspect=None,
     nmax=None,
     color_dict=None,
-    dinc=None,
     lkeys=None,
     bstr_dict=None,
     rotation=None,
@@ -1155,7 +1165,6 @@ def plot_as_array_3d(
     fs=None,
     dcolorbar=None,
     dleg=None,
-    connect=None,
 ):
 
     # --------------
@@ -1423,7 +1432,7 @@ def plot_as_array_3d(
             ref=refZ,
             data=key,
             dtype=datatype,
-            ax=kax,
+            axes=kax,
             ind=0,
         )
 
@@ -1457,7 +1466,7 @@ def plot_as_array_3d(
                 ref=refY,
                 data=keyY,
                 dtype='ydata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
             coll.add_mobile(
@@ -1466,7 +1475,7 @@ def plot_as_array_3d(
                 ref=refX,
                 data=keyX,
                 dtype='xdata',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
             km = f'm{ii:02.0f}'
@@ -1476,7 +1485,7 @@ def plot_as_array_3d(
                 ref=[refX, refY],
                 data=[keyX, keyY],
                 dtype=['xdata', 'ydata'],
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1505,7 +1514,7 @@ def plot_as_array_3d(
                 data=[key, key],
                 dtype=['xdata', 'xdata'],
                 group_vis='X',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1521,7 +1530,7 @@ def plot_as_array_3d(
                 data=keyY,
                 dtype='ydata',
                 group_vis='Y',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1549,7 +1558,7 @@ def plot_as_array_3d(
                 data=[key, key],
                 dtype=['ydata', 'ydata'],
                 group_vis='Y',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1565,7 +1574,7 @@ def plot_as_array_3d(
                 data=keyX,
                 dtype='xdata',
                 group_vis='X',
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1591,7 +1600,7 @@ def plot_as_array_3d(
                 ref=(refX, refY),
                 data=[key, key],
                 dtype=['ydata', 'ydata'],
-                ax=kax,
+                axes=kax,
                 ind=ii,
             )
 
@@ -1606,7 +1615,7 @@ def plot_as_array_3d(
             ref=(refZ,),
             data=keyZ,
             dtype='xdata',
-            ax=kax,
+            axes=kax,
             ind=0,
         )
 
@@ -1666,19 +1675,4 @@ def plot_as_array_3d(
             bstr_dict=bstr_dict,
         )
 
-    # --------------------------
-    # add axes and interactivity
-
-    # add axes
-    for kax in dax.keys():
-        coll.add_axes(key=kax, **dax[kax])
-
-    # increment dict
-
-    coll.setup_interactivity(kinter='inter0', dgroup=dgroup, dinc=dinc)
-
-    # connect
-    if connect is True:
-        coll.connect()
-
-    return coll
+    return coll, dax, dgroup
